/* This file is part of turtle, a turtle-graphics system for Scheme.

Copyright (C) 2008  Luca Saiu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdbool.h>
#include "turtle.h"
#include "guileturtle.h"
#include <guile/gh.h>
//#include <guile/readline.h>

/* The one and only turtle visible from Scheme: */
static turtle_turtle_t turtle;

static void turtle_redraw(void){
  if(turtle->hidden)
    turtle_refresh();
  else{
    /* Refreshing with the cursor is slow, so we only do
       this if really needed (i.e. if the turtle is not
       hidden): */
    turtle_draw_cursor(turtle);
    turtle_refresh_with_cursor();
  }
  //printf("!\n"); fflush(stdout);
}

static SCM turtle_refresh_guile(void){
  turtle_redraw();
  return SCM_UNSPECIFIED;
}

static SCM turtle_forward_guile(SCM length_){
  const float length = scm_to_double(length_);
  turtle_forward(turtle, length);
  turtle_redraw();
  return SCM_UNSPECIFIED;
}

static SCM turtle_backward_guile(SCM length_){
  const float length = scm_to_double(length_);
  turtle_forward(turtle, length);
  turtle_redraw();
  return SCM_UNSPECIFIED;
}

static SCM turtle_left_radiants_guile(SCM angle_){
  const float angle = scm_to_double(angle_);
  turtle_left_radiants(turtle, angle);
  turtle_redraw();
  return SCM_UNSPECIFIED;
}

static SCM turtle_left_degrees_guile(SCM angle_){
  const float angle = scm_to_double(angle_);
  turtle_left_degrees(turtle, angle);
  turtle_redraw();
  return SCM_UNSPECIFIED;
}

static SCM turtle_right_radiants_guile(SCM angle_){
  const float angle = scm_to_double(angle_);
  turtle_right_radiants(turtle, angle);
  turtle_redraw();
  return SCM_UNSPECIFIED;
}

static SCM turtle_right_degrees_guile(SCM angle_){
  const float angle = scm_to_double(angle_);
  turtle_right_degrees(turtle, angle);
  turtle_redraw();
  return SCM_UNSPECIFIED;
}

static SCM turtle_pen_guile(SCM pen_){
  /* Any value different from #f is considered true in Scheme;
     it's ugly, but that's what the Standard says. */
  const bool pen = scm_to_bool(pen_);
  turtle_set_pen(turtle, true);
  turtle_redraw();
  return SCM_UNSPECIFIED;
}

static SCM turtle_pen_up_guile(void){
  turtle_set_pen(turtle, false);
  turtle_redraw();
  return SCM_UNSPECIFIED;
}

static SCM turtle_pen_down_guile(void){
  turtle_set_pen(turtle, true);
  turtle_redraw();
  return SCM_UNSPECIFIED;
}

static SCM turtle_home_guile(void){
  turtle_home(turtle);
  turtle_redraw();
  return SCM_UNSPECIFIED;
}

static SCM turtle_hide_guile(void){
  turtle_hide(turtle);
  turtle_redraw();
  return SCM_UNSPECIFIED;
}

static SCM turtle_show_guile(void){
  turtle_show(turtle);
  turtle_redraw();
  return SCM_UNSPECIFIED;
}

static SCM turtle_clear_guile(void){
  turtle_home(turtle);
  turtle_clear_window(turtle_black);
  turtle_redraw();
  return SCM_UNSPECIFIED;
}

static SCM turtle_goto_guile(SCM x_, SCM y_){
  const float x = scm_to_double(x_);
  const float y = scm_to_double(y_);
  turtle_goto(turtle, x, y);
  turtle_redraw();
  return SCM_UNSPECIFIED;
}

void turtle_initialize_guile_support(void){
  /* Initialize Guile: */
  scm_init_guile();
  
  ///* Let's make the command line more comfortable: */
  //scm_init_readline();
  
  
  /* Immediately make the window: */
  turtle_initialize_graphics(TURTLE_WINDOW_WIDTH, TURTLE_WINDOW_HEIGHT);
  turtle_clear_window(turtle_black);
  
  /* Set the expose event handler: */
  turtle_set_expose_event_handler(turtle_refresh);

  /* Make the turtle: */
  turtle = turtle_make_turtle();
  
  /* Make all primitives visible from Guile: */
  scm_c_define_gsubr("forward", 1, 0, 0, turtle_forward_guile);
  scm_c_define_gsubr("backward", 1, 0, 0, turtle_backward_guile);
  scm_c_define_gsubr("move", 1, 0, 0, turtle_forward_guile);

  scm_c_define_gsubr("left", 1, 0, 0, turtle_left_degrees_guile);
  scm_c_define_gsubr("left-radiants", 1, 0, 0, turtle_left_radiants_guile);
  scm_c_define_gsubr("left-degrees", 1, 0, 0, turtle_left_degrees_guile);

  scm_c_define_gsubr("right", 1, 0, 0, turtle_right_degrees_guile);
  scm_c_define_gsubr("right-radiants", 1, 0, 0, turtle_right_radiants_guile);
  scm_c_define_gsubr("right-degrees", 1, 0, 0, turtle_right_degrees_guile);

  scm_c_define_gsubr("pen-up", 0, 0, 0, turtle_pen_up_guile);
  scm_c_define_gsubr("pen-down", 0, 0, 0, turtle_pen_down_guile);
  scm_c_define_gsubr("pen", 1, 0, 0, turtle_pen_guile);

  scm_c_define_gsubr("home", 0, 0, 0, turtle_home_guile);
  scm_c_define_gsubr("goto", 2, 0, 0, turtle_goto_guile);
  scm_c_define_gsubr("go-to", 2, 0, 0, turtle_goto_guile);

  scm_c_define_gsubr("hide", 0, 0, 0, turtle_hide_guile);
  scm_c_define_gsubr("show", 0, 0, 0, turtle_show_guile);

  scm_c_define_gsubr("clear", 0, 0, 0, turtle_clear_guile);
  scm_c_define_gsubr("reset", 0, 0, 0, turtle_clear_guile);

  scm_c_define_gsubr("refresh", 0, 0, 0, turtle_refresh_guile);
  scm_c_define_gsubr("redraw", 0, 0, 0, turtle_refresh_guile);

  /* Show the cursor: */
  turtle_refresh_guile();
}
